#!/usr/bin/env python

"""
Phrase iterators providing navigation over common positions for a number of
different terms.

Copyright (C) 2009 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from itermerge import itermerge
from bisect import insort_right

class PhraseIterator(itermerge):

    "Iteration over many terms."

    def __init__(self, sequences):
        itermerge.__init__(self, sequences)

    def _add_seq(self, sequence, i):

        "Store the details of the given 'sequence' at position 'i'."

        insort_right(self.iters, (len(sequence), i, iter(sequence)))

    def next(self):
        if self.iters:
            while 1:
                freq, i, it = self.iters[0]
                doc, positions = it.next()
                values = [(i, positions)]
                for freq, i, it in self.iters[1:]:
                    positions = it.from_document(doc)
                    if positions is None:
                        break
                    else:
                        insort_right(values, (i, positions))
                else:
                    return doc, [positions for (i, positions) in values]
        else:
            raise StopIteration

    def close(self):
        for freq, i, it in self.iters:
            if hasattr(it, "close"):
                it.close()
        self.iters = []

# vim: tabstop=4 expandtab shiftwidth=4
